String x_label = "Checkout Frequency";
String y_label = "Checkout Frequency";

// Layout variables 
int margin = 100;
float plotW = width - margin*2;
float plotH = height - margin*2;

// interaction variables
String x_variable = "age", y_variable = "full_frequency", color_representation = "bookType";
float hoverDistance = 30;

float max_age, max_relative_age, max_fullFrequency, max_partialFrequency = 0;
float min_age, min_relative_age, min_fullFrequency, min_partialFrequency = 10000;
int min_publishOrder = 1000, max_publishOrder = 0;

float valX, valY, start1_X, stop1_X, start2_X, stop2_X, start1_Y, stop1_Y, start2_Y, stop2_Y = 0;



Table table;
int numRows, numCols;

DataPoint[] dataPoints;

void setup() {
  size(1440, 900);
  smooth();
  //frameRate(60);
  surface.setResizable(true); // make the screen resizable
  
  table = loadTable("final_cleaned.csv", "header"); // put data into Processing table from csv
  numRows = table.getRowCount(); // get number of data points
  numCols = table.getColumnCount(); // get number of attributes per data point
  
  // create data points and fill array
  dataPoints = new DataPoint[numRows+1]; // initialize data points array length
  int counter = 0;
  for (TableRow row : table.rows()){
    counter +=1;
    createDataPoint(row, counter); 
  }
}

void draw() {
  background(25, 76, 95);
  
  for(int i=1; i<dataPoints.length; i++){
    checkXVariable(i);
    checkYVariable(i);
    // //checkColor(i);
    // map values to x and y axis
    float posX = map(valX, start1_X, stop1_X, start2_X, stop2_X);
    float posY = map(valY, start1_Y, stop1_Y, start2_Y, stop2_Y);
    
    if(mouseX >= posX - hoverDistance && mouseX <= posX + hoverDistance 
    && mouseY >= posY - hoverDistance && mouseY <= posY + hoverDistance){
      
      dataPoints[i].displayLabel();
      if(color_representation == "bookType"){
        String bookType = dataPoints[i].itemType;
        dataPoints[i].setColor(i, bookType);
      }
    } else { dataPoints[i].setColor(i, "null"); }
    
    dataPoints[i].setPosition(posX, posY);
    dataPoints[i].display();
  }
  
  drawBoarders();
  drawTitles();
  drawAxis();
  drawInformation();
  drawCredits();
}

void drawBoarders(){
  fill(48, 99, 118);
  rect(0,0,margin,height);
  rect(0,0,width,margin);
  rect(0,height-margin,width,margin);
  rect(width-margin,margin,margin,height);
}

void drawTitles(){
  textAlign(CENTER, CENTER);
  textSize(18);
  fill(200);
  text("Book Age (days): " + x_label, width/2, start2_Y + 40);
  
  textAlign(CENTER, CENTER);
  pushMatrix();
  rotate(-PI/2);
  text("Checkout Frequency (count): " + y_label, -height/2,margin-60);
  popMatrix();
}

void drawAxis(){
  
  float x_axis_distance = (stop2_X - start2_X) / 10;
  float y_axis_distance = (start2_Y - stop2_Y) / 10;
  
  float x_axis_value = (stop1_X - start1_X) / 10;
  float y_axis_value = (stop1_Y - start1_Y) / 10;
  
  // print x axis
  for(int k=1; k<11; k++){
   textAlign(CENTER, CENTER);
   textSize(12);
   fill(200);
   text((int)(k*x_axis_value), k*x_axis_distance, start2_Y+10);
  }
  
  // print y axis
  for(int l=1; l<11; l++){
  textAlign(CENTER, CENTER);
  textSize(12);
  fill(200);
  text((int)(l*y_axis_value), margin-10, start2_Y - (l*y_axis_distance));
  }
  
}

void drawInformation(){
  fill(140, 197, 161);
  textSize(32);
  //rotate(.5);
  //translate(-100, -1*height/1.5);
  textAlign(LEFT, CENTER);
  text("Are Books Still Prevelant in an e-Media World?", margin, margin/2);
  
  fill(127, 163, 177);
  textSize(14);
  textAlign(LEFT, CENTER);
  if(x_variable.equals("age")){ x_label = "Age"; text("X-Variable: Age", width-4*margin, margin/3); }
  else {x_label = "Relative Age"; text("X-Variable: Relative Age", width - 4*margin, margin/3);}
  
  if(y_variable.equals("full_frequency")){ y_label = "Total"; text("Y-Variable: Total Checkout Frequency", width - 4*margin, margin/2); }
  else {y_label = "Relative"; text("Y-Variable: Relative Checkout Frequency", width - 4*margin, margin/2);}
  
  if(color_representation.equals("bookType")){
    text("Color Range: Book Type", width - 4*margin, 2*margin/3);
    //rect();
    //text();
  }
  else{ text("Color Range: Publication Date", width - 4*margin, 2*margin/3); }
}

void drawCredits(){
  fill(140, 197, 161);
  textSize(12);
  textAlign(LEFT, CENTER);
  text("Thomas Hervey", width-margin, height - 2*margin/3);
  text("MAT 259", width-margin, height - margin/2);
  text("Winter 2016", width-margin, height - margin/3);
}